package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.item.totem.TotemItem;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.network.PacketBuffer;
import net.minecraft.server.management.PlayerList;
import net.minecraft.util.Hand;
import net.minecraft.util.text.StringTextComponent;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fml.network.NetworkEvent;

import java.util.function.Supplier;

public class ChangeTargetPacket {
	private final Hand hand;
	
	public ChangeTargetPacket(final Hand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ChangeTargetPacket packet, final PacketBuffer buffer) {
		Network.writeVersionInfo(buffer, false);
		buffer.writeEnum(packet.hand);
	}
	
	public static ChangeTargetPacket decode(final PacketBuffer buffer) {
		Network.checkVersion(buffer);
		return new ChangeTargetPacket(buffer.readEnum(Hand.class));
	}
	
	public static void handle(final ChangeTargetPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayerEntity sender = context.getSender();
			if (sender == null) { return; }
			
			ItemStack totemStack = sender.getItemInHand(packet.hand);
			PlayerList playerList = TotemOfReviving.PROXY.getServer().getPlayerList();
			int targetIndex = TotemItem.getTargetIndex(totemStack) + 1;
			
			if (targetIndex > playerList.getPlayerCount()-1) { targetIndex = 0; }
			ServerPlayerEntity target = playerList.getPlayers().get(targetIndex);
			
			TotemItem.setTargetIndex(totemStack, targetIndex);
			TotemItem.setTargetUUID(totemStack, target.getUUID());
			TotemItem.setTargetName(totemStack, target.getDisplayName().getString());
			TotemItem.setTargetDeaths(totemStack, target);
			
			sender.sendMessage(new StringTextComponent(TextFormatting.WHITE + "Now targetting " + TextFormatting.GRAY + target.getDisplayName().getString() + "."), sender.getUUID());
		});
		context.setPacketHandled(true);
	}
}
