package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.item.totem.TotemItem;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.network.PacketBuffer;
import net.minecraft.util.Hand;
import net.minecraft.util.text.StringTextComponent;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fml.network.NetworkEvent;

import java.util.function.Supplier;

public class ReviveTargetPacket {
	private final Hand hand;
	
	public ReviveTargetPacket(final Hand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ReviveTargetPacket packet, final PacketBuffer buffer) {
		Network.writeVersionInfo(buffer, false);
		buffer.writeEnum(packet.hand);
	}
	
	public static ReviveTargetPacket decode(final PacketBuffer buffer) {
		Network.checkVersion(buffer);
		return new ReviveTargetPacket(buffer.readEnum(Hand.class));
	}
	
	public static void handle(final ReviveTargetPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayerEntity sender = context.getSender();
			if (sender == null) { return; }
			
			ItemStack totemStack = sender.getItemInHand(packet.hand);
			ServerPlayerEntity target;
			
			try {
				target = TotemOfReviving.PROXY.getServer().getPlayerList().getPlayer(TotemItem.getTargetUUID(totemStack));
				if (target == null) { throw new NullPointerException("Target is null!"); }
			} catch (NullPointerException exception) {
				sender.sendMessage(new StringTextComponent(TextFormatting.WHITE + "Unable to find player!"), sender.getUUID());
				return;
			}
			
			if (!target.isSpectator()) {
				sender.sendMessage(new StringTextComponent(TextFormatting.GRAY + target.getDisplayName().getString() + TextFormatting.WHITE + " is not dead!"), sender.getUUID());
				return;
			}
			
			if (!target.getLevel().equals(sender.getLevel()) && !TotemItem.canReviveAcrossDimensions(totemStack)) {
				sender.sendMessage(new StringTextComponent(TextFormatting.GRAY + target.getDisplayName().getString() + TextFormatting.WHITE + " is in a different dimension!"), sender.getUUID());
				return;
			}
			
			TotemItem.setTargetDeaths(totemStack, target);
			if (!TotemItem.canTotemAffordTarget(totemStack)) {
				sender.sendMessage(new StringTextComponent(TextFormatting.WHITE + "Not enough charge!"), sender.getUUID());
				return;
			}
			
			target.teleportTo(sender.getLevel(), sender.position().x, sender.position().y, sender.position().z, sender.yRot, sender.xRot);
			target.setGameMode(sender.getLevel().getServer().getDefaultGameType());
			TotemItem.setCharge(totemStack, TotemItem.getCharge(totemStack) - TotemItem.getTargetCost(totemStack));
			totemStack.hurtAndBreak(1, sender, e -> e.broadcastBreakEvent(packet.hand));
			sender.sendMessage(new StringTextComponent(TextFormatting.WHITE + "Successfully revived " + TextFormatting.GRAY + target.getDisplayName().getString() + "!"), sender.getUUID());
		});
		context.setPacketHandled(true);
	}
}
